import os
import pandas as pd

from datetime import datetime

from nsys_recipe.lib import helpers
from nsys_recipe.lib import recipe
from nsys_recipe.lib.args import Option
from nsys_recipe.lib.recipe import Context

class Dx12MemOps(recipe.Recipe):
    display_name = 'DX12 Memory Operations'
    description = """
    This recipe identifies memory operations with the following warnings:
    1. HEAP_CREATED_WITH_ZEROING
    2. COMMITTED_RESOURCE_CREATED_WITH_ZEROING
    3. NONEMPTY_MAP_FROM_UPLOAD_HEAP
    4. NONEMPTY_MAP_TO_WRITE_COMBINE_PAGE
    5. NONEMPTY_UNMAP_TO_READBACK_HEAP
    6. NONEMPTY_UNMAP_FROM_WRITE_BACK_PAGE
    7. READ_FROM_UPLOAD_HEAP_SUBRESOURCE
    8. READ_FROM_SUBRESOURCE_TO_WRITE_COMBINE_PAGE
    9. WRITE_TO_READBACK_HEAP_SUBRESOURCE
    10. WRITE_TO_SUBRESOURCE_FROM_WRITE_BACK_PAGE
"""

    @staticmethod
    def mapper_func(nsysrep, parsed_args):
        sqlite_file = helpers.nsysrep_to_sqlite(nsysrep)
        if sqlite_file is None:
            return None

        stats_cls = helpers.get_stats_cls('dx12_mem_ops', 'Dx12MemOps')
        return helpers.stats_cls_to_df(sqlite_file, parsed_args, stats_cls)

    def reducer_func(self, dfs):
        dfs = helpers.filter_none(dfs)
        df = pd.concat(dfs, ignore_index=True)

        # Remove any tags or hidden columns that are for internal use.
        df.columns = df.columns.str.replace('(:).*', '', regex=True)
        df = df.loc[:, ~df.columns.str.startswith('_')]

        df.to_parquet(self.add_output_file('analysis.parquet'))

    def save_metadata(self):
        self._analysis_dict.update({
            'EndTime': str(datetime.now()),
            'InputReports': self._parsed_args.dir,
            'Outputs': self._output_files
        })
        self.create_analysis_file()

    def run(self, context):
        super().run(context)
        mapper_res = context.wait(context.map(
            self.mapper_func,
            self._parsed_args.dir,
            parsed_args=self._parsed_args
        ))
        self.reducer_func(mapper_res)

        self.create_notebook('analysis.ipynb', 'nsys_display.py')
        self.save_metadata()

    @classmethod
    def get_argument_parser(cls):
        parser = super().get_argument_parser()

        parser.add_recipe_argument(Option.OUTPUT)
        parser.add_recipe_argument(Option.FORCE_OVERWRITE)
        parser.add_recipe_argument(Option.START)
        parser.add_recipe_argument(Option.END)
        parser.add_recipe_argument(Option.NVTX)
        parser.add_recipe_argument(Option.ROWS)
        parser.add_recipe_argument(Option.REPORT_DIR, required=True)

        return parser
